@@ -3,6 +3,7 @@ from datetime import datetime, date, time
 from uuid import uuid4, UUID
 import re
 
+import pony
 from pony.utils import is_utf8, decorator, throw, localbase
 from pony.converting import str2date, str2datetime
 from pony.orm.ormtypes import LongStr, LongUnicode
@@ -72,7 +73,6 @@ class DBAPIProvider(object):
     quote_char = '"'
     max_params_count = 200
     max_name_len = 128
-    ignore_none = True
     table_if_not_exists_syntax = True
     index_if_not_exists_syntax = True
     max_time_precision = default_time_precision = 6
@@ -95,6 +95,7 @@ class DBAPIProvider(object):
         provider.inspect_connection(connection)
         provider.release(connection)
 
+    @wrap_dbapi_exceptions
     def inspect_connection(provider, connection):
         pass
 
@@ -162,6 +163,9 @@ class DBAPIProvider(object):
             return quote_char + name + quote_char
         return '.'.join(provider.quote_name(item) for item in name)
 
+    def normalize_vars(provider, vars, vartypes):
+        pass
+
     def ast2sql(provider, ast):
         builder = provider.sqlbuilder_cls(provider, ast)
         return builder.sql, builder.adapter
@@ -174,12 +178,41 @@ class DBAPIProvider(object):
         return provider.pool.connect()
 
     @wrap_dbapi_exceptions
-    def release(provider, connection):
-        return provider.pool.release(connection)
+    def set_transaction_mode(provider, connection, cache):
+        pass
+
+    @wrap_dbapi_exceptions
+    def commit(provider, connection):
+        core = pony.orm.core
+        if core.debug: core.log_orm('COMMIT')
+        connection.commit()
+
+    @wrap_dbapi_exceptions
+    def rollback(provider, connection):
+        core = pony.orm.core
+        if core.debug: core.log_orm('ROLLBACK')
+        connection.rollback()
+
+    @wrap_dbapi_exceptions
+    def release(provider, connection, cache=None):
+        core = pony.orm.core
+        if cache is not None and cache.db_session is not None and cache.db_session.ddl:
+            provider.drop(connection)
+        else:
+            if core.debug: core.log_orm('RELEASE CONNECTION')
+            provider.pool.release(connection)
 
     @wrap_dbapi_exceptions
     def drop(provider, connection):
-        return provider.pool.drop(connection)
+        core = pony.orm.core
+        if core.debug: core.log_orm('CLOSE CONNECTION')
+        provider.pool.drop(connection)
+
+    @wrap_dbapi_exceptions
+    def disconnect(provider):
+        core = pony.orm.core
+        if core.debug: core.log_orm('DISCONNECT')
+        provider.pool.disconnect()
 
     @wrap_dbapi_exceptions
     def execute(provider, cursor, sql, arguments=None, returning_id=False):
@@ -191,14 +224,6 @@ class DBAPIProvider(object):
             else: cursor.execute(sql, arguments)
             if returning_id: return cursor.lastrowid
 
-    @wrap_dbapi_exceptions
-    def commit(provider, connection):
-        connection.commit()
-
-    @wrap_dbapi_exceptions
-    def rollback(provider, connection):
-        connection.rollback()
-
     converter_classes = []
 
     def _get_converter_type_by_py_type(provider, py_type):
@@ -219,19 +244,13 @@ class DBAPIProvider(object):
     def get_pool(provider, *args, **kwargs):
         return Pool(provider.dbapi_module, *args, **kwargs)
 
-    def set_transaction_mode(provider, connection, optimistic):
-        pass
-
-    def start_optimistic_save(provider, connection):
-        pass
-
-    def table_exists(provider, connection, table_name):
+    def table_exists(provider, connection, table_name, case_sensitive=True):
         throw(NotImplementedError)
 
-    def index_exists(provider, connection, table_name, index_name):
+    def index_exists(provider, connection, table_name, index_name, case_sensitive=True):
         throw(NotImplementedError)
 
-    def fk_exists(provider, connection, table_name, fk_name):
+    def fk_exists(provider, connection, table_name, fk_name, case_sensitive=True):
         throw(NotImplementedError)
 
     def table_has_data(provider, connection, table_name):
@@ -240,10 +259,10 @@ class DBAPIProvider(object):
         cursor.execute('SELECT 1 FROM %s LIMIT 1' % table_name)
         return cursor.fetchone() is not None
 
-    def disable_fk_checks_if_necessary(provider, connection):
+    def disable_fk_checks(provider, connection):
         pass
 
-    def enable_fk_checks_if_necessary(provider, connection, prev_state):
+    def enable_fk_checks(provider, connection, prev_state):
         pass
 
     def drop_table(provider, connection, table_name):
@@ -259,8 +278,11 @@ class Pool(localbase):
         pool.kwargs = kwargs
         pool.con = None
     def connect(pool):
+        core = pony.orm.core
         if pool.con is None:
+            if core.debug: core.log_orm('GET NEW CONNECTION')
             pool.con = pool.dbapi_module.connect(*pool.args, **pool.kwargs)
+        elif core.debug: core.log_orm('GET CONNECTION FROM THE LOCAL POOL')
         return pool.con
     def release(pool, con):
         assert con is pool.con
@@ -269,7 +291,7 @@ class Pool(localbase):
             pool.drop(con)
             raise
     def drop(pool, con):
-        assert con is pool.con
+        assert con is pool.con, (con, pool.con)
         pool.con = None
         con.close()
     def disconnect(pool):
