@@ -9,7 +9,7 @@ from uuid import UUID
 import cx_Oracle
 
 from pony.orm import core, sqlbuilding, dbapiprovider, sqltranslation
-from pony.orm.core import log_orm, log_sql, DatabaseError
+from pony.orm.core import log_orm, log_sql, DatabaseError, TranslationError
 from pony.orm.dbschema import DBSchema, DBObject, Table, Column
 from pony.orm.dbapiprovider import DBAPIProvider, wrap_dbapi_exceptions, get_version_tuple
 from pony.utils import throw
@@ -19,25 +19,31 @@ class OraTable(Table):
         result = Table.get_objects_to_create(table, created_tables)
         for column in table.column_list:
             if column.is_pk == 'auto':
-                sequence = OraSequence(table)
+                sequence_name = column.converter.attr.kwargs.get('sequence_name')
+                sequence = OraSequence(table, sequence_name)
                 trigger = OraTrigger(table, column, sequence)
                 result.extend((sequence, trigger))
                 break
         return result
 
 class OraSequence(DBObject):
-    def __init__(sequence, table):
+    typename = 'Sequence'
+    def __init__(sequence, table, name=None):
         sequence.table = table
         table_name = table.name
-        if isinstance(table_name, basestring): sequence.name = table_name + '_SEQ'
+        if name is not None: sequence.name = name
+        elif isinstance(table_name, basestring): sequence.name = table_name + '_SEQ'
         else: sequence.name = tuple(table_name[:-1]) + (table_name[0] + '_SEQ',)
-    def exists(sequence, provider, connection):
+    def exists(sequence, provider, connection, case_sensitive=True):
+        if case_sensitive: sql = 'SELECT sequence_name FROM all_sequences ' \
+                                 'WHERE sequence_owner = :so and sequence_name = :sn'
+        else: sql = 'SELECT sequence_name FROM all_sequences ' \
+                    'WHERE sequence_owner = :so and upper(sequence_name) = upper(:sn)'
         owner_name, sequence_name = provider.split_table_name(sequence.name)
         cursor = connection.cursor()
-        cursor.execute('SELECT 1 FROM all_sequences '
-                       'WHERE sequence_owner = :so and sequence_name = :sn',
-                       dict(so=owner_name, sn=sequence_name))
-        return cursor.fetchone() is not None
+        cursor.execute(sql, dict(so=owner_name, sn=sequence_name))
+        row = cursor.fetchone()
+        return row[0] if row is not None else None
     def get_create_command(sequence):
         schema = sequence.table.schema
         seq_name = schema.provider.quote_name(sequence.name)
@@ -54,6 +60,7 @@ BEGIN
 END;""".strip()
 
 class OraTrigger(DBObject):
+    typename = 'Trigger'
     def __init__(trigger, table, column, sequence):
         trigger.table = table
         trigger.column = column
@@ -61,14 +68,18 @@ class OraTrigger(DBObject):
         table_name = table.name
         if not isinstance(table_name, basestring): table_name = table_name[-1]
         trigger.name = table_name + '_BI' # Before Insert
-    def exists(trigger, provider, connection):
+    def exists(trigger, provider, connection, case_sensitive=True):
+        if case_sensitive: sql = 'SELECT trigger_name FROM all_triggers ' \
+                                 'WHERE table_name = :tbn AND table_owner = :o ' \
+                                 'AND trigger_name = :trn AND owner = :o'
+        else: sql = 'SELECT trigger_name FROM all_triggers ' \
+                    'WHERE table_name = :tbn AND table_owner = :o ' \
+                    'AND upper(trigger_name) = upper(:trn) AND owner = :o'
         owner_name, table_name = provider.split_table_name(trigger.table.name)
         cursor = connection.cursor()
-        cursor.execute('SELECT 1 FROM all_triggers '
-                       'WHERE table_name = :tbn AND table_owner = :o '
-                       'AND trigger_name = :trn AND owner = :o',
-                       dict(tbn=table_name, trn=trigger.name, o=owner_name))
-        return cursor.fetchone() is not None
+        cursor.execute(sql, dict(tbn=table_name, trn=trigger.name, o=owner_name))
+        row = cursor.fetchone()
+        return row[0] if row is not None else None
     def get_create_command(trigger):
         schema = trigger.table.schema
         quote_name = schema.provider.quote_name
@@ -114,6 +125,30 @@ class OraBuilder(sqlbuilding.SQLBuilder):
         if returning is not None:
             result.extend((' RETURNING ', builder.quote_name(returning), ' INTO :new_id'))
         return result
+    def SELECT_FOR_UPDATE(builder, nowait, *sections):
+        assert not builder.indent
+        last_section = sections[-1]
+        if last_section[0] != 'LIMIT':
+            return builder.SELECT(*sections), 'FOR UPDATE NOWAIT\n' if nowait else 'FOR UPDATE\n'
+
+        from_section = sections[1]
+        assert from_section[0] == 'FROM'
+        if len(from_section) > 2: throw(NotImplementedError,
+            'Table joins are not supported for Oracle queries which have both FOR UPDATE and ROWNUM')
+
+        order_by_section = None
+        for section in sections:
+            if section[0] == 'ORDER_BY': order_by_section = section
+
+        table_ast = from_section[1]
+        assert len(table_ast) == 3 and table_ast[1] == 'TABLE'
+        table_alias = table_ast[0]
+        rowid = [ 'COLUMN', table_alias, 'ROWID' ]
+        sql_ast = [ 'SELECT', sections[0], [ 'FROM', table_ast ], [ 'WHERE', [ 'IN', rowid,
+                    ('SELECT', [ 'ROWID', ['AS', rowid, 'row-id' ] ]) + sections[1:] ] ] ]
+        if order_by_section: sql_ast.append(order_by_section)
+        result = builder(sql_ast)
+        return result, 'FOR UPDATE NOWAIT\n' if nowait else 'FOR UPDATE\n'
     def SELECT(builder, *sections):
         last_section = sections[-1]
         limit = offset = None
@@ -123,16 +158,24 @@ class OraBuilder(sqlbuilding.SQLBuilder):
             sections = sections[:-1]
         result = builder.subquery(*sections)
         indent = builder.indent_spaces * builder.indent
+
+        if sections[0][0] == 'ROWID':
+            indent0 = builder.indent_spaces
+            x = 't."row-id"'
+        else:
+            indent0 = ''
+            x = 't.*'
+            
         if not limit: pass
         elif not offset:
-            result = [ 'SELECT * FROM (\n' ]
+            result = [ indent0, 'SELECT * FROM (\n' ]
             builder.indent += 1
             result.extend(builder.subquery(*sections))
             builder.indent -= 1
-            result.extend((indent, ') WHERE ROWNUM <= ', builder(limit)))
+            result.extend((indent, ') WHERE ROWNUM <= ', builder(limit), '\n'))
         else:
             indent2 = indent + builder.indent_spaces
-            result = [ 'SELECT * FROM (\n', indent2, 'SELECT t.*, ROWNUM "row-num" FROM (\n' ]
+            result = [ indent0, 'SELECT %s FROM (\n' % x, indent2, 'SELECT t.*, ROWNUM "row-num" FROM (\n' ]
             builder.indent += 2
             result.extend(builder.subquery(*sections))
             builder.indent -= 2
@@ -142,17 +185,19 @@ class OraBuilder(sqlbuilding.SQLBuilder):
                 total_limit = [ 'VALUE', limit[1] + offset[1] ]
                 result.extend(('WHERE ROWNUM <= ', builder(total_limit), '\n'))
             else: result.extend(('WHERE ROWNUM <= ', builder(limit), ' + ', builder(offset), '\n'))
-            result.extend((indent, ') WHERE "row-num" > ', builder(offset)))
+            result.extend((indent, ') t WHERE "row-num" > ', builder(offset), '\n'))
         if builder.indent:
             indent = builder.indent_spaces * builder.indent
             return '(\n', result, indent + ')'
         return result
+    def ROWID(builder, *expr_list):
+        return builder.ALL(*expr_list)
     def LIMIT(builder, limit, offset=None):
         assert False
-        if not offset: return 'LIMIT ', builder(limit), '\n'
-        else: return 'LIMIT ', builder(limit), ' OFFSET ', builder(offset), '\n'
     def DATE(builder, expr):
         return 'TRUNC(', builder(expr), ')'
+    def RANDOM(builder):
+        return 'dbms_random.value'
 
 class OraBoolConverter(dbapiprovider.BoolConverter):
     def sql2py(converter, val):
@@ -185,12 +230,17 @@ class OraStrConverter(dbapiprovider.StrConverter):
             val = val.read()
             if converter.utf8: return val
             val = val.decode('utf8')
-        assert isinstance(val, unicode)
-        val = val.encode(converter.encoding, 'replace')
+        if isinstance(val, unicode):
+            val = val.encode(converter.encoding, 'replace')
         return val
     sql_type = _string_sql_type
 
 class OraIntConverter(dbapiprovider.IntConverter):
+    def init(self, kwargs):
+        dbapiprovider.IntConverter.init(self, kwargs)
+        sequence_name = kwargs.pop('sequence_name', None)
+        if sequence_name is not None and not (self.attr.auto and self.attr.is_pk):
+            throw(TypeError, "Parameter 'sequence_name' can be used only for PrimaryKey attributes with auto=True")
     def sql_type(converter):
         return 'NUMBER(38)'
 
@@ -235,6 +285,20 @@ class OraProvider(DBAPIProvider):
 
     name_before_table = 'owner'
 
+    converter_classes = [
+        (bool, OraBoolConverter),
+        (unicode, OraUnicodeConverter),
+        (str, OraStrConverter),
+        ((int, long), OraIntConverter),
+        (float, OraRealConverter),
+        (Decimal, OraDecimalConverter),
+        (buffer, OraBlobConverter),
+        (datetime, OraDatetimeConverter),
+        (date, OraDateConverter),
+        (UUID, OraUuidConverter),
+    ]
+
+    @wrap_dbapi_exceptions
     def inspect_connection(provider, connection):
         cursor = connection.cursor()
         cursor.execute('SELECT version FROM product_component_version '
@@ -254,6 +318,25 @@ class OraProvider(DBAPIProvider):
     def normalize_name(provider, name):
         return name[:provider.max_name_len].upper()
 
+    def normalize_vars(provider, vars, vartypes):
+        for name, value in vars.iteritems():
+            if value == '':
+                vars[name] = None
+                vartypes[name] = NoneType
+
+    @wrap_dbapi_exceptions
+    def set_transaction_mode(provider, connection, cache):
+        assert not cache.in_transaction
+        db_session = cache.db_session
+        if db_session is not None and db_session.serializable:
+            cursor = connection.cursor()
+            sql = 'SET TRANSACTION ISOLATION LEVEL SERIALIZABLE'
+            if core.debug: log_orm(sql)
+            cursor.execute(sql)
+        cache.immediate = True
+        if db_session is not None and (db_session.serializable or db_session.ddl):
+            cache.in_transaction = True
+
     @wrap_dbapi_exceptions
     def execute(provider, cursor, sql, arguments=None, returning_id=False):
         if type(arguments) is list:
@@ -271,19 +354,6 @@ class OraProvider(DBAPIProvider):
             if arguments is None: cursor.execute(sql)
             else: cursor.execute(sql, arguments)
 
-    converter_classes = [
-        (bool, OraBoolConverter),
-        (unicode, OraUnicodeConverter),
-        (str, OraStrConverter),
-        ((int, long), OraIntConverter),
-        (float, OraRealConverter),
-        (Decimal, OraDecimalConverter),
-        (buffer, OraBlobConverter),
-        (datetime, OraDatetimeConverter),
-        (date, OraDateConverter),
-        (UUID, OraUuidConverter),
-    ]
-
     def get_pool(provider, *args, **kwargs):
         user = password = dsn = None
         if len(args) == 1:
@@ -308,30 +378,39 @@ class OraProvider(DBAPIProvider):
         kwargs.setdefault('increment', 1)
         return OraPool(**kwargs)
 
-    def table_exists(provider, connection, table_name):
+    def table_exists(provider, connection, table_name, case_sensitive=True):
         owner_name, table_name = provider.split_table_name(table_name)
         cursor = connection.cursor()
-        cursor.execute('SELECT 1 FROM all_tables WHERE owner = :o AND table_name = :tn',
-                       dict(o=owner_name, tn=table_name))
-        return cursor.fetchone() is not None
+        if case_sensitive: sql = 'SELECT table_name FROM all_tables WHERE owner = :o AND table_name = :tn'
+        else: sql = 'SELECT table_name FROM all_tables WHERE owner = :o AND upper(table_name) = upper(:tn)'
+        cursor.execute(sql, dict(o=owner_name, tn=table_name))
+        row = cursor.fetchone()
+        return row[0] if row is not None else None
 
-    def index_exists(provider, connection, table_name, index_name):
+    def index_exists(provider, connection, table_name, index_name, case_sensitive=True):
         owner_name, table_name = provider.split_table_name(table_name)
         if not isinstance(index_name, basestring): throw(NotImplementedError)
+        if case_sensitive: sql = 'SELECT index_name FROM all_indexes WHERE owner = :o ' \
+                                 'AND index_name = :i AND table_owner = :o AND table_name = :t'
+        else: sql = 'SELECT index_name FROM all_indexes WHERE owner = :o ' \
+                    'AND upper(index_name) = upper(:i) AND table_owner = :o AND table_name = :t'
         cursor = connection.cursor()
-        cursor.execute('SELECT 1 FROM all_indexes WHERE owner = :o '
-                       'AND index_name = :i AND table_owner = :o AND table_name = :t',
-                       dict(o=owner_name, i=index_name, t=table_name))
-        return cursor.fetchone() is not None
+        cursor.execute(sql, dict(o=owner_name, i=index_name, t=table_name))
+        row = cursor.fetchone()
+        return row[0] if row is not None else None
 
-    def fk_exists(provider, connection, table_name, fk_name):
+    def fk_exists(provider, connection, table_name, fk_name, case_sensitive=True):
         owner_name, table_name = provider.split_table_name(table_name)
         if not isinstance(fk_name, basestring): throw(NotImplementedError)
+        if case_sensitive:
+            sql = "SELECT constraint_name FROM user_constraints WHERE constraint_type = 'R' " \
+                  'AND table_name = :tn AND constraint_name = :cn AND owner = :o'
+        else: sql = "SELECT constraint_name FROM user_constraints WHERE constraint_type = 'R' " \
+                    'AND table_name = :tn AND upper(constraint_name) = upper(:cn) AND owner = :o'
         cursor = connection.cursor()
-        cursor.execute("SELECT 1 FROM user_constraints WHERE constraint_type = 'R' "
-                       'AND table_name = :tn AND constraint_name = :cn AND owner = :o',
-                       dict(tn=table_name, cn=fk_name, o=owner_name))
-        return cursor.fetchone() is not None
+        cursor.execute(sql, dict(tn=table_name, cn=fk_name, o=owner_name))
+        row = cursor.fetchone()
+        return row[0] if row is not None else None
 
     def table_has_data(provider, connection, table_name):
         table_name = provider.quote_name(table_name)
@@ -370,6 +449,7 @@ class OraPool(object):
     def __init__(pool, **kwargs):
         pool._pool = cx_Oracle.SessionPool(**kwargs)
     def connect(pool):
+        if core.debug: log_orm('GET CONNECTION')
         con = pool._pool.acquire()
         con.outputtypehandler = output_type_handler
         return con
