@@ -26,7 +26,7 @@ class DBSchema(object):
     def order_tables_to_create(schema):
         tables = []
         created_tables = set()
-        tables_to_create = set(schema.tables.values())
+        tables_to_create = sorted(schema.tables.itervalues(), key=lambda table: table.name)
         while tables_to_create:
             for table in tables_to_create:
                 if table.parent_tables.issubset(created_tables):
@@ -47,8 +47,29 @@ class DBSchema(object):
         created_tables = set()
         for table in schema.order_tables_to_create():
             for db_object in table.get_objects_to_create(created_tables):
-                if not db_object.exists(provider, connection):
-                    db_object.create(provider, connection)
+                name = db_object.exists(provider, connection, case_sensitive=False)
+                if name is None: db_object.create(provider, connection)
+                elif name != db_object.name:
+                    quote_name = schema.provider.quote_name
+                    n1, n2 = quote_name(db_object.name), quote_name(name)
+                    tn1, tn2 = db_object.typename, db_object.typename.lower()
+                    throw(DBSchemaError, '%s %s cannot be created, because %s %s ' \
+                                         '(with a different letter case) already exists in the database. ' \
+                                         'Try to delete %s %s first.' % (tn1, n1, tn2, n2, n2, tn2))
+    def check_tables(schema, provider, connection):
+        cursor = connection.cursor()
+        for table in sorted(schema.tables.itervalues(), key=lambda table: table.name):
+            if isinstance(table.name, tuple): alias = table.name[-1]
+            elif isinstance(table.name, basestring): alias = table.name
+            else: assert False
+            sql_ast = [ 'SELECT',
+                        [ 'ALL', ] + [ [ 'COLUMN', alias, column.name ] for column in table.column_list ],
+                        [ 'FROM', [ alias, 'TABLE', table.name ] ],
+                        [ 'WHERE', [ 'EQ', [ 'VALUE', 0 ], [ 'VALUE', 1 ] ] ]
+                      ]
+            sql, adapter = provider.ast2sql(sql_ast)
+            if core.debug: log_sql(sql)
+            provider.execute(cursor, sql)
 
 class DBObject(object):
     def create(table, provider, connection):
@@ -58,6 +79,7 @@ class DBObject(object):
         provider.execute(cursor, sql)
 
 class Table(DBObject):
+    typename = 'Table'
     def __init__(table, name, schema):
         if name in schema.tables:
             throw(DBSchemaError, "Table %r already exists in database schema" % name)
@@ -81,8 +103,8 @@ class Table(DBObject):
         if isinstance(table_name, tuple):
             table_name = '.'.join(table_name)
         return '<Table(%s)>' % table_name
-    def exists(table, provider, connection):
-        return provider.table_exists(connection, table.name)
+    def exists(table, provider, connection, case_sensitive=True):
+        return provider.table_exists(connection, table.name, case_sensitive)
     def get_create_command(table):
         schema = table.schema
         case = schema.case
@@ -96,13 +118,13 @@ class Table(DBObject):
             cmd.append(schema.indent + column.get_sql() + ',')
         if len(table.pk_index.columns) > 1:
             cmd.append(schema.indent + table.pk_index.get_sql() + ',')
-        for index in table.indexes.values():
+        for index in sorted(table.indexes.itervalues(), key=lambda index: index.name):
             if index.is_pk: continue
             if not index.is_unique: continue
             if len(index.columns) == 1: continue
             cmd.append(schema.indent+index.get_sql() + ',')
         if not schema.named_foreign_keys:
-            for foreign_key in table.foreign_keys.values():
+            for foreign_key in sorted(table.foreign_keys.itervalues(), key=lambda fk: fk.name):
                 if schema.inline_fk_syntax and len(foreign_key.child_columns) == 1: continue
                 cmd.append(schema.indent+foreign_key.get_sql() + ',')
         cmd[-1] = cmd[-1][:-1]
@@ -111,24 +133,24 @@ class Table(DBObject):
     def get_objects_to_create(table, created_tables=None):
         if created_tables is None: created_tables = set()
         result = [ table ]
-        for index in table.indexes.values():
+        for index in sorted(table.indexes.itervalues(), key=lambda index: index.name):
             if index.is_pk or index.is_unique: continue
             assert index.name is not None
             result.append(index)
         schema = table.schema
         if schema.named_foreign_keys:
-            for foreign_key in table.foreign_keys.values():
+            for foreign_key in sorted(table.foreign_keys.itervalues(), key=lambda fk: fk.name):
                 if foreign_key.parent_table not in created_tables: continue
                 result.append(foreign_key)
             for child_table in table.child_tables:
                 if child_table not in created_tables: continue
-                for foreign_key in child_table.foreign_keys.values():
+                for foreign_key in sorted(child_table.foreign_keys.itervalues(), key=lambda fk: fk.name):
                     if foreign_key.parent_table is not table: continue
                     result.append(foreign_key)
         created_tables.add(table)
         return result
-    def add_column(table, column_name, sql_type, is_not_null=None):
-        return table.schema.column_class(column_name, table, sql_type, is_not_null)
+    def add_column(table, column_name, sql_type, converter, is_not_null=None, sql_default=None):
+        return table.schema.column_class(column_name, table, sql_type, converter, is_not_null, sql_default)
     def add_index(table, index_name, columns, is_pk=False, is_unique=None, m2m=False):
         assert index_name is not False
         if index_name is True: index_name = None
@@ -149,7 +171,7 @@ class Table(DBObject):
 
 class Column(object):
     auto_template = '%(type)s PRIMARY KEY AUTOINCREMENT'
-    def __init__(column, name, table, sql_type, is_not_null=None):
+    def __init__(column, name, table, sql_type, converter, is_not_null=None, sql_default=None):
         if name in table.column_dict:
             throw(DBSchemaError, "Column %r already exists in table %r" % (name, table.name))
         table.column_dict[name] = column
@@ -157,7 +179,9 @@ class Column(object):
         column.table = table
         column.name = name
         column.sql_type = sql_type
+        column.converter = converter
         column.is_not_null = is_not_null
+        column.sql_default = sql_default
         column.is_pk = False
         column.is_pk_part = False
         column.is_unique = False
@@ -181,6 +205,9 @@ class Column(object):
             else:
                 if column.is_unique: append(case('UNIQUE'))
                 if column.is_not_null: append(case('NOT NULL'))
+        if column.sql_default not in (None, True, False):
+            append(case('DEFAULT'))
+            append(column.sql_default)
         if schema.inline_fk_syntax and not schema.named_foreign_keys:
             foreign_key = table.foreign_keys.get((column,))
             if foreign_key is not None:
@@ -202,6 +229,7 @@ class Constraint(DBObject):
         constraint.name = name
 
 class Index(Constraint):
+    typename = 'Index'
     def __init__(index, name, table, columns, is_pk=False, is_unique=None):
         assert len(columns) > 0
         for column in columns:
@@ -232,8 +260,8 @@ class Index(Constraint):
         index.columns = columns
         index.is_pk = is_pk
         index.is_unique = is_unique
-    def exists(index, provider, connection):
-        return provider.index_exists(connection, index.table.name, index.name)
+    def exists(index, provider, connection, case_sensitive=True):
+        return provider.index_exists(connection, index.table.name, index.name, case_sensitive)
     def get_sql(index):
         return index._get_create_sql(inside_table=True)
     def get_create_command(index):
@@ -266,6 +294,7 @@ class Index(Constraint):
         return ' '.join(cmd)
 
 class ForeignKey(Constraint):
+    typename = 'Foreign key'
     def __init__(foreign_key, name, child_table, child_columns, parent_table, parent_columns, index_name):
         schema = parent_table.schema
         if schema is not child_table.schema: throw(DBSchemaError,
@@ -299,8 +328,8 @@ class ForeignKey(Constraint):
                 if columns[:child_columns_len] == child_columns: break
             else: child_table.add_index(index_name, child_columns, is_pk=False,
                                         is_unique=False, m2m=bool(child_table.m2m))
-    def exists(foreign_key, provider, connection):
-        return provider.fk_exists(connection, foreign_key.child_table.name, foreign_key.name)
+    def exists(foreign_key, provider, connection, case_sensitive=True):
+        return provider.fk_exists(connection, foreign_key.child_table.name, foreign_key.name, case_sensitive)
     def get_sql(foreign_key):
         return foreign_key._get_create_sql(inside_table=True)
     def get_create_command(foreign_key):
