@@ -6,7 +6,7 @@ options.CUT_TRACEBACK = False
 
 from pony.orm.core import *
 
-sql_debug(True)
+sql_debug(False)
 
 db = Database('sqlite', 'inheritance1.sqlite', create_db=True)
 
@@ -45,7 +45,11 @@ class Course(db.Entity):
 
 db.generate_mapping(create_tables=True)
 
+@db_session
 def populate_database():
+    if Person.select().first():
+        return # already populated
+
     p = Person(name='Person1', ssn='SSN1')
     g = Group(number=123)
     prof = Professor(name='Professor1', salary=1000, position='position1', ssn='SSN5')
@@ -56,17 +60,27 @@ def populate_database():
     commit()
 
 def show_all_persons():
-    for obj in select(p for p in Person):
+    for obj in Person.select():
         print obj
-        print obj._attrs_
         for attr in obj._attrs_:
             print attr.name, "=", attr.__get__(obj)
         print
 
 if __name__ == '__main__':
-    # populate_database()
+    populate_database()
     # show_all_persons()
-    s1 = Student.get(name='Student1')
-    mentor = s1.mentor
-    print isinstance(mentor, Assistant)
-    print mentor.name
+
+    sql_debug(True)
+
+    with db_session:
+        s1 = Student.get(name='Student1')
+        if s1 is None:
+            print 'Student1 not found'
+        else:
+            mentor = s1.mentor
+            print mentor.name, 'is mentor of Student1'
+            print 'Is he assistant?', isinstance(mentor, Assistant)
+        print
+
+        for s in Student.select(lambda s: s.mentor.salary == 1000):
+            print s.name
