@@ -36,6 +36,10 @@ from test_frames import *
 from test_core_multiset import *
 from test_core_find_in_cache import *
 from test_db_session import *
+from test_lazy import *
+from test_query import *
+from test_filter import *
+from test_crud import *
 
 #from new_tests import *
 
